const { existsSync, readFileSync, writeFileSync,ensureDirSync } = require("fs-extra");
const { join } = require("path");
const { EOL } = require("os");
const HOOK_MARK = "##### CREATED BY UCF-SCRIPTS #####";
const cwd = process.cwd();
const { spawn } = require("child_process");
const preCommitConfig = {
    commitLint: true,
};

//commit-msg模板
function getPreCommitTemplate() {
    return [
        "npx --no-install commitlint --edit $1",
        " ",
        HOOK_MARK,
    ].join(EOL);
}

// 安装 Husky&添加commit-msg钩子
function installhusky() {
    const installHusky = spawn("npx", ["husky", "install"]);
    installHusky.on("close", (code) => {
        if (code === 0) {
            console.log("Husky installed successfully");
            // 添加 commit-msg 钩子
            const addCommitMsgHook = spawn("npx", [
                "husky",
                "add",
                ".husky/commit-msg",
                getPreCommitTemplate(),
            ]);
            addCommitMsgHook.on("close", (code) => {
                if (code === 0) {
                    console.log("commit-msg hook added successfully");
                } else {
                    console.error("Failed to add commit-msg hook");
                }
            });
        } else {
            console.error("Failed to install Husky");
        }
    });
}

createCommitLint = () =>{
    const cwd = process.cwd();
    let str = `
module.exports = {
    extends: ["./node_modules/ucf-scripts/commitlint.config.js"],
};
    `
    const hookPath = join(cwd, "commitlint.config.js");

    const existHooks = existsSync(hookPath);
    if(!existHooks){
        writeFileSync(hookPath, str, "utf8");
    }
}

function install() {
    const usePreCommit = preCommitConfig.commitLint;
    const hookPath = join(cwd, ".husky");
    const preCommitHooks = join(hookPath, "commit-msg");
    const existHooks = existsSync(preCommitHooks);
    const isUcfPreCommit = existHooks && readFileSync(preCommitHooks, "utf8").includes(HOOK_MARK);
    // 判断用户是否有commit-msg
    if (usePreCommit && existHooks && isUcfPreCommit) {
        console.log(
            "目前.husky目录下面已经存在 commit-msg 钩子. ucf-scripts内置 commit-msg 钩子不会生效."
        );
        return;
    }
    //用户没有，就帮他创建一个
    if (usePreCommit && !existHooks) {
        createCommitLint();
        installhusky();
        console.log("Create commit-msg hook");
    }
}

module.exports = {
    name: "init",
    excute: () => {
        install();
    },
};
