const chalk = require("chalk");
const path = require("path");
const fse = require("fs-extra");
const unzipper = require("unzipper");
const rp = require("request-promise");
const cliProgress = require("cli-progress");
const argv = require("minimist")(process.argv.slice(2));
const download = async function (options, filename, cb) {
    let opts = {
        method: "get",
        headers: {
            Accept: "text/html,application/xhtml+xml,application/xml;q=0.9,image/webp,image/apng,*/*;q=0.8,application/signed-exchange;v=b3",
            "Accept-Encoding": "gzip, deflate",
            "Accept-Language": "zh-CN,zh;q=0.9,en;q=0.8,zh-TW;q=0.7",
            Connection: "keep-alive",
            "Upgrade-Insecure-Requests": 1,
        },
    };
    opts = { ...opts, ...options };
    // 获得文件夹路径
    let fileFolder = path.dirname(filename);
    // 创建文件夹
    fse.ensureDirSync(fileFolder);
    // 开始下载无需返回
    rp(opts).pipe(fse.createWriteStream(filename)).on("close", cb);
};

/**
 * 下载zip压缩包包含路径文件名
 */
const getRemoteZip = () => {
    let url =
        "https://git.yonyou.com/yonyou-ide/tinper-next-packages/next-webapp/-/archive/master/next-webapp-master.zip";
    return new Promise((resolve, reject) => {
        download({ url }, `next-webapp.tmp`, () => {
            resolve({ success: true });
        });
    });
};

init = async (folderName) => {
    console.log();
    console.log(
        chalk.cyan.bold(`[Info] :    🚀 Start downloading UCF project to the current directory 🎁`)
    );
    console.log(chalk.cyan.bold(`Path:${path.resolve(".", folderName)}  🏠`));
    console.log();
    const bar = new cliProgress.SingleBar({
        format: "{bar} {percentage}%",
        barCompleteChar: "\u2588",
        barIncompleteChar: "\u2591",
        hideCursor: true,
        barStyle: {
            bg: "blue", // 设置进度条背景色为蓝色
            fg: "green", // 设置进度条前景色为白色
        },
    });
    if (!fse.existsSync(folderName)) {
        let total = 100;
        let process = 50;
        bar.start(total, process); // 设定最大值为100，当前值为50
        for (let i = 0; i < 100; i++) {
            bar.update(i); // 更新进度条
        }
        let result = await getRemoteZip({
            filename: folderName,
        });
        let filepath = path.resolve(".");
        if (result.success) {
            bar.update(100);
            bar.stop(); // 停止进度条
            fse.createReadStream(`${filepath}/next-webapp.tmp`)
                .pipe(unzipper.Extract({ path: filepath }))
                .on("close", () => {
                    // 删除压缩包
                    fse.remove(`${filepath}/next-webapp.tmp`);
                    fse.renameSync(`${filepath}/next-webapp-master`, `${filepath}/${folderName}`);
                });
            console.log();
            console.log();
            console.log(chalk.cyan(`🚀 Next, install NPM package dependencies 🎁 `));
            console.log(chalk.cyan(`[Tips] : 🏆  cd ${folderName} && npm install && npm start`));
        }
    } else {
        console.log(chalk.red.bold(`[Error] :   ⚠️  directory ${folderName} already exists. 😫`));
        console.log(chalk.yellow(`[Tips] :    🤔 Try renaming the project name 🤗  `));
        process.exit(0);
    }
};

module.exports = {
    name: "init",
    description: "这是初始化命令",
    execute() {
        let mode = argv.mode;
        if(mode){
            switch (mode) {
                //初始化commit-msg
                case "commitlint":
                    let commit_init = require('../commit/index');
                    commit_init.install();
                    break;
                default :
                    break;
            }
        }else{
            //初始化项目
            let args = argv._.slice(1)
            let folderName = args[1] || "next-webapp";
            init(folderName);
        }
    },
};
