const { ESLint } = require('eslint');
const argv = require("minimist")(process.argv.slice(2));
const commands = argv;
const util = require("../../util");
const cfg = util.getUcfConfig(commands._);
const path = require('path')
// 获取配置文件的绝对路径
const configPath = path.resolve(__dirname, '../../../.eslintrc');
const scan_root = util.defaultExtendFunc(cfg.scan_root, "ucf-apps");

function createESLintInstance(){
  return new ESLint({ useEslintrc: false, overrideConfigFile: configPath, fix: true });
}

async function lintAndFix(eslint, filePaths) {
  const results = await eslint.lintFiles(filePaths);
  await ESLint.outputFixes(results);

  const formatter = await eslint.loadFormatter('stylish');
  const resultsText = formatter.format(results)
  return resultsText;
}

function outputLintingResults(results) {
  if (results) {
    console.log("Linting errors found!");
    console.log(results);
  } else {
    console.log("No linting errors found.");
  }
  return results;
}

async function lintFiles(filePaths) {
    const eslint = createESLintInstance();
    const results = await lintAndFix(eslint, filePaths);
    return outputLintingResults(results);
}


//启动构建
module.exports = {
  name: "lint",
  description: "这是lint命令",
  check: false,
  execute: () => {
        // 指定要检查的文件或目录
        const filesPath = [`${scan_root}/**/*.js`];
        // 调用函数来执行ESLint检查
        lintFiles(filesPath);
    },
};

