const { default: chalk } = require("chalk");
const util = require("../../util");
const argv = require("minimist")(process.argv.slice(2));
const commands = argv;
const { readFileSync, writeFileSync }  = require('fs-extra');
const cfg = util.getUcfConfig(commands._);
const v2OldOptions  = require('../check/v2OldOptions');
var _prettier = require("prettier");
const prettierConfig = {
    "endOfLine": "lf",
    "semi": true,
    "singleQuote": true,
    "tabWidth": 4,
    "trailingComma": "all",
    "arrowParens": "avoid"
}
  
//获取配置项的文本
const getUcfOptionMap = {};
let objectLeftRight = {
    left: "{",
    right: "}",
};

let arrayLeftRight = {
    left: "[",
    right: "]",
};

let normalObj = {
    type: "normal",
};
const oldOptionsMap = {
    css: objectLeftRight,
    less: objectLeftRight,
    HtmlPluginConf: objectLeftRight,
    splitChunks: objectLeftRight,
    postcss_plugins: arrayLeftRight,
    babel_include: arrayLeftRight,
    babel_presets: arrayLeftRight,
    babel_plugins: arrayLeftRight,
    loader: arrayLeftRight,
    devPlugins: arrayLeftRight,
    buildPlugins: arrayLeftRight,
    hasHash: normalObj,
    open_source_map: normalObj,
    res_extra: normalObj,
};


function getExtractedContent(paramter, data) {
    let { key, left, right } = paramter;
    // 使用正则表达式匹配 css 后面的部分
    const pattern = new RegExp(`${key}\\s*:([\\s\\S]*)`);
    const match = pattern.exec(data);
    let res = "";
    if (match) {
        const content = `${match[0]}`;
        const counter = { openBrackets: 0, closeBrackets: 0 };
        // 遍历匹配结果，找到第一个 { 和最后一个 } 的位置
        for (let i = 0; i < content.length; i++) {
            if (paramter.type === "normal") {
                if (content[i] === ",") {
                    res = content.substring(-1, i + 1);
                    break;
                }
            } else {
                if (content[i] === left) {
                    counter.openBrackets++;
                } else if (content[i] === right) {
                    counter.closeBrackets++;
                    if (counter.openBrackets === counter.closeBrackets) {
                        res = content.substring(-1, i + 1) + ",";
                        // console.log('提取到的内容：', res);
                        break;
                    }
                }
            }
        }
    }
    return res;
}

//删除多余旧的api
deleteOption = (data) =>{
    let content = data ;
    Object.keys(getUcfOptionMap).forEach((ucfItem) => {
        let replaceStr = getUcfOptionMap[ucfItem];
        content = content.replace(replaceStr, "");
        //处理历史模板有两个属性的重复写法
        content = content.replace(replaceStr, "");
    });
    return content
}

getValueText = (key) =>{
    let userLoaders = getUcfOptionMap[key]
    var newContent = userLoaders.replace(new RegExp(`${key}\s*:`), "");
    // 检查字符串最后一个字符是否是逗号
    if (newContent.charAt(newContent.length - 1) === ",") {
        // 删除最后一个字符
        newContent = newContent.substring(0, newContent.length - 1);
    }
    return newContent
}

getIsComma  = (content) =>{
    let count = 0;
    let char = '';
    for (let i = content.length - 1; i < content.length; i--) {
      if (content[i] !== ' ' && content[i] !== '\n') {
        count++;
        if (count === 3) {
          char = content[i];
          break;
        }
      }
    }
    return char === ','
}

//整合更新新添加的api
addOptions = (data) =>{
    let loaderConfigList = ["less","postcss_plugins","babel_include","css","babel_presets"]
    loaderConfigList = loaderConfigList.filter(item => !util.isEmpty(cfg[item]) && cfg[item] !== undefined )
    let loaderInfo = loaderConfigList.join('、');

    let loaderStr = cfg?.loader && cfg.loader.length ? `defaultLoaderOptions.loaders = ${getValueText('loader')}` : ''
    let loaderConfig = `,
    loaderConfig(defaultLoaderOptions){
        //${loaderInfo}需要手动按照升级文档修改一下
        //可参照文档v2升级v3废弃API兼容
        ${loaderStr}
        return defaultLoaderOptions
    },`
    let pluginConfig =`
    pluginConfig(defaultPluginOptions){
        ${cfg.HtmlPluginConf ? "//HtmlPluginConf需要手动按照升级文档修改一下" : ""}
        defaultPluginOptions.plugins = argv[0] === "build" ? ${getValueText('buildPlugins')} : ${getValueText('devPlugins')}
        return defaultPluginOptions
    }`
    return `${loaderConfig}${pluginConfig},\n`
}

updatePrettierContent = async(text) =>{
    try {
        const formatText = await _prettier.format(text, {
            parser: 'babel',
            ...prettierConfig,
        });
        return formatText
    } catch (error) {
        console.log('格式化代码出现问题，请手动处理代码格式！')
        return text
    }
    
}

//更新ucf.config.js
updateUcfCfg = async() =>{
    let filePath = util.getRunPath("ucf.config.js");
    const data = readFileSync(filePath, 'utf8');
    writeFileSync(util.getRunPath('ucf.config.backup.js'), data,'utf8');
    let list = util.checkUcfConfigOptions(cfg)
    list.map((item) => {
        getUcfOptionMap[item] = getExtractedContent({ key: item, ...oldOptionsMap[item] }, data);
    });
    let sta = addOptions(data)

    //删除旧的属性
    let content = deleteOption(data)
    let newContent = getSecondLastIndex(content,"}",sta)
    newContent = await updatePrettierContent(newContent)
    await writeFileSync(util.getRunPath('ucf.config.js'), newContent,'utf8');
    console.log(`Successfully updated ${filePath}`);
    console.log('')
    console.log(chalk.green('ucf.config.js已经完成初步升级修改，可能会有部分问题，可按照下面提示继续修改！'))
    console.log('');
    util.getOfficialUrl()
    let options = util.checkUcfConfigOptions(cfg);
    let otherOptions = ['open_source_map','res_extra','babel_plugins','loader']
    options = options.filter(it=>!util.isEmpty(cfg[it]))
    console.log(chalk.bgGreen('具体修改可参考下面或者官方文档！'))
    options.forEach(it=>{
        if(otherOptions.indexOf(it) === -1){
            console.log(chalk.yellow(`${it}:`));
            console.log(chalk.bgGreen(`升级前用法：`))
            console.log(`${v2OldOptions[it].old}`)
            console.log((`--------------------------------------------------------`));
            console.log(chalk.bgGreen(`升级之后用法：`))
            console.log(`${v2OldOptions[it].new}`)
            console.log('');
        }
    })
    console.log('已经备份原来的ucf.config.js文件，更名为ucf.config.backup.js');
    console.log('请参照上述提示或者官方文档修改')
    console.log(chalk.green('升级完成请将ucf.config.backup.js文件删除掉！'))
}

getSecondLastIndex = (str,target,newStr) =>{
    const lastOccurrenceIndex = str.lastIndexOf(target);
    const index = str.lastIndexOf(
        target,
        lastOccurrenceIndex - 1
      );
    const modifiedStr = `${str.slice(0, index)}${newStr}${str.slice(index)}`;
    return modifiedStr;
}

/**
 * update 主程序
 */
update = () => {
    try {
        updateUcfCfg()
    } catch (error) {
        console.log(error)
        console.log(chalk.red('升级过程中出现未知错误，请参考官网文档手动修改！'))
        util.getOfficialUrl()
    }
};

module.exports = {
    name: "update",
    description: "这是升级命令",
    execute: () => {
        update();
    },
};




