/* Base Webpack4 config
 * @Author:             Kvkens(yueming@yonyou.com)
 * @Date:               2019-01-21 13:02:27
 * @Last Modified by:   Kvkens
 * @Last Modified time: 2019-10-17 18:57:29
 */

const path = require("path");
const webpack = require("webpack");
const MiniCssExtractPlugin = require("mini-css-extract-plugin");
const { getDefaultRuleOptions } = require("./default");
const { TNSClientPlugin } = require("@tinper/next-plugin");
const argv = require("minimist")(process.argv.slice(2));
const commands = argv;
const util = require("../src/util");
const cfg = util.getUcfConfig(commands._);
const { enabled: tnsEnabled, ...tnsDetailCfg } = cfg.tnsCfg || {};
const { hash, ruleConfig, presetConfig } = cfg;

let _context = "";

// 处理上下文路径
if (cfg.context) {
    _context = `${cfg.context}/`;
}

// 输出资源的文件夹
const dist_root = util.defaultExtendFunc(cfg.dist_root, "ucf-publish");

let hashJsname = util.handleChangeOptions("hash", {
    key: "js",
    ext: "js",
    cfgOpt: hash,
});

let hashCssname = util.handleChangeOptions("hash", {
    key: "css",
    ext: "css",
    cfgOpt: hash,
});

let defaultRuleOptions = getDefaultRuleOptions(cfg,_context,commands._[0] === 'start');
let rules = [];
// ruleConfig配置
let userRuleOptions = util.configHandler(ruleConfig, defaultRuleOptions);
// presetConfig特征开关配置
rules = ['babel','css','less','img','icon']
    .filter((ruleItem) => presetConfig === undefined || presetConfig[ruleItem] !== false)
    .map((ruleItem) => {
        const rule = userRuleOptions[ruleItem] || defaultRuleOptions[ruleItem];
        if (presetConfig?.ts === false && ruleItem === "babel") {
            return {
                ...rule,
                test: /\.m?jsx?$/,
            };
        }
        return rule;
    });
const config = {
    output: {
        path: path.resolve('.', dist_root, _context),
        filename: hashJsname,
        chunkFilename: hashJsname,
        publicPath: cfg.publicPath==true ? '/' + _context : cfg.publicPath,
        environment: {
            // The environment supports arrow functions ('() => { ... }').
            arrowFunction: false,
            // The environment supports BigInt as literal (123n).
            bigIntLiteral: false,
            // The environment supports const and let for variable declarations.
            const: false,
            // The environment supports destructuring ('{ a, b } = obj').
            destructuring: false,
            // The environment supports an async import() function to import EcmaScript modules.
            dynamicImport: false,
            // The environment supports 'for of' iteration ('for (const x of array) { ... }').
            forOf: false,
            // The environment supports ECMAScript Module syntax to import ECMAScript modules (import ... from '...').
            module: false,
        },
    },
    module: {
        rules: util.defaultExtendFunc(defaultRuleOptions.loaders,rules,),
    },
    resolve: {
        extensions: util.defaultExtendFunc(cfg.extensions, [
            ".ts",
            ".tsx",
            ".jsx",
            ".js",
            ".less",
            ".css",
            ".json",
        ]),
        alias: util.defaultExtendFunc(cfg.alias, {
            'ucf-apps': path.resolve('.', 'ucf-apps/'),
            'ucf-common': path.resolve('.', 'ucf-common/src/'),
            components: path.resolve('.', 'ucf-common/src/components/'),
            static: path.resolve('.', 'ucf-common/src/static/'),
            utils: path.resolve('.', 'ucf-common/src/utils/')
        }),
    },
    externals: util.defaultExtendFunc(
        cfg.externals,
        //tns默认排除以下包
        tnsEnabled
            ? {
                  react: "React",
                  "react-dom": "ReactDOM",
                  "@tinper/next-ui": "TinperNext",
              }
            : {}
    ),
    plugins: [
        new webpack.BannerPlugin({
            banner: `File:[file] Date:${new Date()}`,
        }),
        new MiniCssExtractPlugin({
            filename: hashCssname,
            chunkFilename: hashCssname,
            ignoreOrder: true,
        }),
        new webpack.ProgressPlugin(),
        new webpack.DefinePlugin(util.defaultExtendFunc(cfg.global_env, {})),
    ],
};
//tns配置
tnsEnabled &&
    config.plugins.push(
        new TNSClientPlugin({
            libraryDebug: true,
            libraryEnv: "online",
            ...tnsDetailCfg,
        })
    );
module.exports = config;
