const path = require("path");
const util = require("../src/util");
const cloneDeep = require('clone-deep')
const loaderConfigHook = (userConfig, configName, otherOption) => {
    let defaultConfig = cloneDeep(defaultLoaderOptions);
    let oldOption = defaultConfig[configName].options;
    defaultConfig[configName].options = { ...oldOption, ...otherOption };
    let userConfigType = typeof userConfig;
    if (userConfigType === "function") {
        let func = userConfig;
        return func(defaultConfig)[configName];
    }
    return defaultConfig[configName];
};

const defaultLoaderOptions = {
    styleLoader: {
        loader: require("mini-css-extract-plugin").loader,
        options: {
            publicPath: "./",
        },
    },
    cssLoader: {
        loader: require.resolve("css-loader"),
        options: {
            esModule: false, //禁用 ES 模块语法
            modules: {
                auto: true,
                localIdentName: '[name]__[local]--[hash:base64:5]'
            }
        },
    },
    postcssLoader: {
        loader: require.resolve("postcss-loader"),
        options: {
            postcssOptions: {
                plugins: () => [
                    require("autoprefixer")({
                        overrideBrowserslist: [
                            "last 2 Chrome versions",
                            "last 2 Firefox versions",
                            "Safari >= 7",
                            "ie > 10",
                        ],
                    }),
                    require("postcss-flexbugs-fixes"),
                    require("postcss-import"),
                ],
            },
        },
    },
    lessLoader: {
        loader: require.resolve("less-loader"),
        options: {},
    },
    babelLoader: {
        loader: require.resolve("babel-loader"),
        options: {
            babelrc: false,
            sourceType: "unambiguous",
            presets: [
                [
                    require.resolve("@babel/preset-env"),
                    {
                        targets: {
                            edge: "17",
                            firefox: "60",
                            chrome: "67",
                            safari: "11.1",
                            ie: "10",
                        },
                    },
                ],
                "@babel/preset-flow",
                "@babel/preset-react",
                "@babel/preset-typescript",
            ],
            plugins: [
                [
                    require.resolve("@babel/plugin-transform-runtime"),
                    {
                        corejs: false,
                        helpers: true,
                        regenerator: true,
                        useESModules: false,
                    },
                ],
                [
                    require.resolve("babel-plugin-dynamic-import-webpack"),
                    {
                        helpers: false,
                        polyfill: true,
                        regenerator: true,
                    },
                ],
                [
                    require.resolve("@babel/plugin-proposal-decorators"),
                    {
                        legacy: true,
                    },
                ],
                [
                    require.resolve("@babel/plugin-proposal-class-properties"),
                    {
                        loose: true,
                    },
                ],
                [
                    require.resolve("@babel/plugin-proposal-private-methods"),
                    {
                        loose: true,
                    },
                ],
                [
                    require.resolve("@babel/plugin-proposal-private-property-in-object"),
                    {
                        loose: true,
                    },
                ],
                [require.resolve("@babel/plugin-proposal-optional-chaining"), {}, "ucf-changing"], // 支持可选链
                [
                    require.resolve("@babel/plugin-proposal-nullish-coalescing-operator"),
                    {},
                    "ucf-coalescing",
                ], // 支持双问号
                require.resolve("@babel/plugin-proposal-object-rest-spread"),
            ],
        },
    },
    urlLoader: {
        loader: require.resolve("url-loader"),
        options: {
            limit: 8196,
            esModule: false,
            name: "[name].[hash:8].[ext]",
        },
    },
    loaders:[]
};
//默认rule的配置
const getDefaultRuleOptions = (cfg,_publicPath,isStart) => {
    
    let { loaderConfig, limit } = cfg;

    //扫描路径
    const scan_root = util.defaultExtendFunc(cfg.scan_root, "ucf-apps");
    let urlLoaderLimit = util.handleChangeOptions("limit", {
        cfgOpt: limit,
        key: "icon",
    });
    let outputPath = isStart && cfg.publicPath ? `${_publicPath}` :   "" ;
    //返回默认配置项
    return {
        css: {
            test: /\.css$/,
            type: "javascript/auto",
            use: [
                loaderConfigHook(loaderConfig, "styleLoader"),
                loaderConfigHook(loaderConfig, "cssLoader"),
                loaderConfigHook(loaderConfig, "postcssLoader"),
                loaderConfigHook(loaderConfig, "lessLoader"),
            ],
        },
        babel: {
            test: /\.m?(ts|js)x?$/,
            include: [
                path.resolve(".", "ucf-common"),
                path.resolve(".", scan_root),
                // path.resolve('.', 'node_modules/@babel/register'),
                path.resolve(".", "node_modules/ansi-regex"),
                path.resolve(".", "node_modules/strip-ansi"),
                path.resolve(".", "node_modules/ucf-scripts"),
                path.resolve(".", "node_modules/query-string"),
                path.resolve(".", "node_modules/split-on-first"),
                path.resolve(".", "node_modules/strict-uri-encode"),
            ],
            use: [
                loaderConfigHook(loaderConfig, "babelLoader")
                // defaultLoaderOptions.babelLoader
            ],
        },
        less: {
            test: /\.less$/,
            type: "javascript/auto",
            use: [
                loaderConfigHook(loaderConfig, "styleLoader"),
                loaderConfigHook(loaderConfig, "cssLoader"),
                loaderConfigHook(loaderConfig, "postcssLoader"),
                loaderConfigHook(loaderConfig, "lessLoader"),
            ],
        },
        img: {
            test: /\.(png|jpg|jpeg|gif)(\?.+)?$/,
            type: "javascript/auto",
            use: [
                loaderConfigHook(loaderConfig, "urlLoader", {
                    limit: urlLoaderLimit,
                    publicPath: `/${_publicPath}assets/images`,
                    outputPath:outputPath + "assets/images/"
                }),
            ],
        },
        icon: {
            test: /\.(woff|woff2|eot|ttf|svg|svgz|otf)$/,
            use: [
                loaderConfigHook(loaderConfig, "urlLoader", {
                    limit: urlLoaderLimit,
                    publicPath: `/${_publicPath}assets/fonts`,
                    outputPath:outputPath + "assets/fonts/"
                }),
            ],
            type: "javascript/auto",
        },
        loaders:loaderConfigHook(loaderConfig, "loaders"),
    };
};

//默认plugin配置
const defaultPluginOptions = {
    CopyWebpackPlugin: {
        // patterns: [],
    },
    CleanWebpackPlugin: {},
    HtmlWebPackPlugin: {},
    TerserPlugin: {
        // test: /\.js(\?.*)?$/i,
        // cache: true,
        parallel: true,
        extractComments: false // 启用/禁用剥离注释功能

        // include:'vendor',
        // exclude:'vendor',
    },
    CssMinimizerPlugin: {
        minimizerOptions: {
            mergeLonghand: false,
            discardComments: { removeAll: true },
        },
    },
    plugins:[]
};

const pluginsMap = {
    HtmlWebPackPlugin: require("html-webpack-plugin"),
    TerserPlugin: require("terser-webpack-plugin"),
    CssMinimizerPlugin: require("css-minimizer-webpack-plugin"),
    CleanWebpackPlugin: require("clean-webpack-plugin").CleanWebpackPlugin,
    CopyWebpackPlugin: require("copy-webpack-plugin"),
};

module.exports = {
    defaultPluginOptions,
    pluginsMap,
    getDefaultRuleOptions,
};
