/* Start Webpack5 config
 * @Author: Kvkens(yueming@yonyou.com)
 * @Date:   2019-01-21 13:02:27
 * @Last Modified by:   Kvkens
 * @Last Modified time: 2019-05-10 10:45:56
 */

const webpack = require("webpack");
const { merge } = require("webpack-merge");
const argv = require("minimist")(process.argv.slice(2));
const commands = argv;
const util = require("../src/util");
const base = require("./base.config");
const cfg = util.getUcfConfig(commands._);
const OpenBrowserPlugin = require("open-browser-plugin");
const browserHost = cfg.host ? cfg.host : "127.0.0.1";
const protocol = process.argv.includes("--https") ? "https" : "http";
const pluginConfig = cfg.pluginConfig;
let entries = {};
let htmlConfList = []
if (!cfg.useCustomEntries) {
    htmlConfList = util.htmlWebpackPluginHook(cfg, "dev");
    htmlConfList.forEach((htmlItem) => {
        let { entryKey, entryChunk } = htmlItem;
        entries[entryKey] = entryChunk;
    });
} else {
    entries = cfg.useCustomEntries
}
let plugins = [new webpack.HotModuleReplacementPlugin()];
// 判断是否加载默认页面打开浏览器
if (commands.homepage) {
    plugins.push(
        new OpenBrowserPlugin({
            url: `${protocol}://${browserHost}:${commands.port || 3000}/${commands.homepage || ""}`,
        })
    );
}

//默认的配置用于merge操作
const config = {
    entry: entries,
    devtool: util.defaultExtendFunc(cfg.devtool, "source-map"),
    mode: "development",
    plugins: util.defaultExtendFunc(
        [...plugins, ...util.pluginConfigHook(pluginConfig, "HtmlWebPackPlugin", htmlConfList)],
        util.pluginConfigHook(pluginConfig,'plugins')
    ),
};

module.exports = util.configHandler(cfg.webpackConfig, merge(base, config));
